// Fragment shader for per-pixel Phong interpolation and shading.

// The "varying" keyword means that the parameter's value is interpolated
// between the nearby vertices.
varying vec3 N;
varying vec3 v;

uniform float brightness;
uniform float alphaX;
uniform float alphaY;

const int NUM_LIGHTS = 2; // directional and point
const vec3 eps = vec3(0.0, 1.0, 0.0); // arbitrary reference point for determining normal plane basis
const float pi = 3.14159;

void main()
{
    // The scene's ambient light.
    vec4 globalAmbient = gl_LightModel.ambient * gl_FrontMaterial.ambient;



	// The normal vectors is generally not normalized after being
	// interpolated across a triangle.  Here we normalize it.
	vec3 Normal = normalize(N);

	// Since the vertex is in eye space, the direction to the
	// viewer is simply the normalized vector from v to the
	// origin.
	vec3 Viewer = -normalize(v);

    gl_FragColor = globalAmbient + gl_FrontMaterial.emission;

	for (int i = 0; i < NUM_LIGHTS; i++) {
        vec3 Light = normalize(gl_LightSource[i].position.xyz - (i == 0 ? vec3(0,0,0) : v));
	    vec3 Half = normalize(Viewer + Light);

        float NdotL = dot(Normal, Light);

        float diffuseShade =  max(NdotL, 0.0);

        vec3 Tangent = normalize(cross(Normal, eps));
        vec3 Bitangent = normalize(cross(Normal, Tangent));
        float beta = -2.0 * (pow(dot(Half, Tangent) / alphaX, 2) + 
                           pow(dot(Half, Bitangent) / alphaY, 2))
                        / (1 + dot(Half, Normal));
        float specularShade = exp(beta) * max(0, sqrt(dot(Normal, Light) / dot(Normal, Viewer))) // trick to keep things above 0, see unity tutorial
                                        / (4 * pi * alphaX * alphaY);
        
	    float a = gl_LightSource[i].constantAttenuation;
	    float b = gl_LightSource[i].linearAttenuation;
	    float c = gl_LightSource[i].quadraticAttenuation;
	    float r = length(vec3(gl_LightSource[i].position - v)); 
	    float attenuatingCoefficient = i == 0 ? 1.0 : 
                                                (1.0 / (a + b*r + c*r*r));

        vec4 diffuse = diffuseShade * gl_FrontLightProduct[i].diffuse;
        vec4 specular = specularShade * gl_FrontLightProduct[i].specular;
        vec4 intensity = (diffuse + specular + gl_FrontLightProduct[i].ambient) * attenuatingCoefficient;
        
        gl_FragColor += intensity;
    }
	gl_FragColor = gl_FragColor * brightness;
}